@@ -46,6 +46,7 @@ int (*register_user)(Client *client);
 int (*tkl_hash)(unsigned int c);
 char (*tkl_typetochar)(int type);
 int (*tkl_chartotype)(char c);
+char (*tkl_configtypetochar)(const char *name);
 const char *(*tkl_type_string)(TKL *tk);
 const char *(*tkl_type_config_string)(TKL *tk);
 char *(*tkl_uhost)(TKL *tkl, char *buf, size_t buflen, int options);
@@ -84,7 +85,6 @@ void (*broadcast_md_client)(ModDataInfo *mdi, Client *client, ModData *md);
 void (*broadcast_md_channel)(ModDataInfo *mdi, Channel *channel, ModData *md);
 void (*broadcast_md_member)(ModDataInfo *mdi, Channel *channel, Member *m, ModData *md);
 void (*broadcast_md_membership)(ModDataInfo *mdi, Client *client, Membership *m, ModData *md);
-int (*check_banned)(Client *client, int exitflags);
 int (*check_deny_version)(Client *client, const char *software, int protocol, const char *flags);
 void (*broadcast_md_client_cmd)(Client *except, Client *sender, Client *acptr, const char *varname, const char *value);
 void (*broadcast_md_channel_cmd)(Client *except, Client *sender, Channel *channel, const char *varname, const char *value);
@@ -120,6 +120,8 @@ TKL *(*find_tkl_banexception)(int type, const char *usermask, const char *hostma
 TKL *(*find_tkl_nameban)(int type, const char *name, int hold);
 TKL *(*find_tkl_spamfilter)(int type, const char *match_string, unsigned short action, unsigned short target);
 int (*find_tkl_exception)(int ban_type, Client *client);
+int (*server_ban_parse_mask)(Client *client, int add, char type, const char *str, char **usermask_out, char **hostmask_out, int *soft, const char **error);
+void (*tkl_added)(Client *client, TKL *tkl);
 int (*is_silenced)(Client *client, Client *acptr);
 int (*del_silence)(Client *client, const char *mask);
 int (*add_silence)(Client *client, const char *mask, int senderr);
@@ -137,6 +139,15 @@ char *(*get_chmodes_for_user)(Client *client, const char *flags);
 WhoisConfigDetails (*whois_get_policy)(Client *client, Client *target, const char *name);
 int (*make_oper)(Client *client, const char *operblock_name, const char *operclass, ConfigItem_class *clientclass, long modes, const char *snomask, const char *vhost);
 int (*unreal_match_iplist)(Client *client, NameList *l);
+void (*webserver_send_response)(Client *client, int status, char *msg);
+void (*webserver_close_client)(Client *client);
+int (*webserver_handle_body)(Client *client, WebRequest *web, const char *readbuf, int length);
+void (*rpc_response)(Client *client, json_t *request, json_t *result);
+void (*rpc_error)(Client *client, json_t *request, JsonRpcError error_code, const char *error_message);
+void (*rpc_error_fmt)(Client *client, json_t *request, JsonRpcError error_code, const char *fmt, ...);
+int (*websocket_handle_websocket)(Client *client, WebRequest *web, const char *readbuf2, int length2, int callback(Client *client, char *buf, int len));
+int (*websocket_create_packet)(int opcode, char **buf, int *len);
+int (*websocket_create_packet_simple)(int opcode, const char **buf, int *len);
 
 Efunction *EfunctionAddMain(Module *module, EfunctionType eftype, int (*func)(), void (*vfunc)(), void *(*pvfunc)(), char *(*stringfunc)(), const char *(*conststringfunc)())
 {
@@ -349,7 +360,6 @@ void efunctions_init(void)
 	efunc_init_function(EFUNC_BROADCAST_MD_CHANNEL, broadcast_md_channel, NULL);
 	efunc_init_function(EFUNC_BROADCAST_MD_MEMBER, broadcast_md_member, NULL);
 	efunc_init_function(EFUNC_BROADCAST_MD_MEMBERSHIP, broadcast_md_membership, NULL);
-	efunc_init_function(EFUNC_CHECK_BANNED, check_banned, NULL);
 	efunc_init_function(EFUNC_INTRODUCE_USER, introduce_user, NULL);
 	efunc_init_function(EFUNC_CHECK_DENY_VERSION, check_deny_version, NULL);
 	efunc_init_function(EFUNC_BROADCAST_MD_CLIENT_CMD, broadcast_md_client_cmd, NULL);
@@ -374,6 +384,7 @@ void efunctions_init(void)
 	efunc_init_function(EFUNC_PARSE_MESSAGE_TAGS, parse_message_tags, &parse_message_tags_default_handler);
 	efunc_init_function(EFUNC_MTAGS_TO_STRING, mtags_to_string, &mtags_to_string_default_handler);
 	efunc_init_function(EFUNC_TKL_CHARTOTYPE, tkl_chartotype, NULL);
+	efunc_init_function(EFUNC_TKL_CONFIGTYPETOCHAR, tkl_configtypetochar, NULL);
 	efunc_init_function(EFUNC_TKL_TYPE_STRING, tkl_type_string, NULL);
 	efunc_init_function(EFUNC_TKL_TYPE_CONFIG_STRING, tkl_type_config_string, NULL);
 	efunc_init_function(EFUNC_CAN_SEND_TO_CHANNEL, can_send_to_channel, NULL);
@@ -391,6 +402,8 @@ void efunctions_init(void)
 	efunc_init_function(EFUNC_FIND_TKL_NAMEBAN, find_tkl_nameban, NULL);
 	efunc_init_function(EFUNC_FIND_TKL_SPAMFILTER, find_tkl_spamfilter, NULL);
 	efunc_init_function(EFUNC_FIND_TKL_EXCEPTION, find_tkl_exception, NULL);
+	efunc_init_function(EFUNC_SERVER_BAN_PARSE_MASK, server_ban_parse_mask, NULL);
+	efunc_init_function(EFUNC_TKL_ADDED, tkl_added, NULL);
 	efunc_init_function(EFUNC_ADD_SILENCE, add_silence, add_silence_default_handler);
 	efunc_init_function(EFUNC_DEL_SILENCE, del_silence, del_silence_default_handler);
 	efunc_init_function(EFUNC_IS_SILENCED, is_silenced, is_silenced_default_handler);
@@ -409,4 +422,13 @@ void efunctions_init(void)
 	efunc_init_function(EFUNC_WHOIS_GET_POLICY, whois_get_policy, NULL);
 	efunc_init_function(EFUNC_MAKE_OPER, make_oper, make_oper_default_handler);
 	efunc_init_function(EFUNC_UNREAL_MATCH_IPLIST, unreal_match_iplist, NULL);
+	efunc_init_function(EFUNC_WEBSERVER_SEND_RESPONSE, webserver_send_response, webserver_send_response_default_handler);
+	efunc_init_function(EFUNC_WEBSERVER_CLOSE_CLIENT, webserver_close_client, webserver_close_client_default_handler);
+	efunc_init_function(EFUNC_WEBSERVER_HANDLE_BODY, webserver_handle_body, webserver_handle_body_default_handler);
+	efunc_init_function(EFUNC_RPC_RESPONSE, rpc_response, rpc_response_default_handler);
+	efunc_init_function(EFUNC_RPC_ERROR, rpc_error, rpc_error_default_handler);
+	efunc_init_function(EFUNC_RPC_ERROR_FMT, rpc_error_fmt, rpc_error_fmt_default_handler);
+	efunc_init_function(EFUNC_WEBSOCKET_HANDLE_WEBSOCKET, websocket_handle_websocket, websocket_handle_websocket_default_handler);
+	efunc_init_function(EFUNC_WEBSOCKET_CREATE_PACKET, websocket_create_packet, websocket_create_packet_default_handler);
+	efunc_init_function(EFUNC_WEBSOCKET_CREATE_PACKET_SIMPLE, websocket_create_packet_simple, websocket_create_packet_simple_default_handler);
 }
