@@ -430,18 +430,19 @@ const char *Module_Create(const char *path_)
 		if (Mod_Handle)
 			*Mod_Handle = mod;
 		irc_dlsym(Mod, "Mod_Test", Mod_Test);
+		/* add module here, so ModuleSetOptions() w/MOD_OPT_PRIORITY is available in Mod_Test() */
+		AddListItemPrio(mod, Modules, 0);
 		if (Mod_Test)
 		{
-			if ((ret = (*Mod_Test)(&mod->modinfo)) < MOD_SUCCESS) {
-				ircsnprintf(errorbuf, sizeof(errorbuf), "Mod_Test returned %i",
-					   ret);
+			if ((ret = (*Mod_Test)(&mod->modinfo)) < MOD_SUCCESS)
+			{
+				ircsnprintf(errorbuf, sizeof(errorbuf), "Mod_Test returned %i", ret);
 				/* We EXPECT the module to have cleaned up its mess */
 				Module_free(mod);
 				return (errorbuf);
 			}
 		}
 		mod->flags = MODFLAG_TESTING;		
-		AddListItemPrio(mod, Modules, 0);
 		return NULL;
 	}
 	else
@@ -560,6 +561,9 @@ void FreeModObj(ModuleObject *obj, Module *m)
 	else if (obj->type == MOBJ_HISTORY_BACKEND) {
 		HistoryBackendDel(obj->object.history_backend);
 	}
+	else if (obj->type == MOBJ_RPC) {
+		RPCHandlerDel(obj->object.rpc);
+	}
 	else
 	{
 		unreal_log(ULOG_FATAL, "module", "FREEMODOBJ_UNKNOWN_TYPE", NULL,
@@ -731,6 +735,8 @@ void module_loadall(void)
 	iFP	fp;
 	Module *mi, *next;
 	
+	loop.config_status = CONFIG_STATUS_LOAD;
+
 	/* Run through all modules and check for module load */
 	for (mi = Modules; mi; mi = next)
 	{
@@ -1216,7 +1222,7 @@ void ModuleSetOptions(Module *module, unsigned int options, int action)
 {
 	unsigned int oldopts = module->options;
 
-	if (options == MOD_OPT_UNLOAD_PRIORITY)
+	if (options == MOD_OPT_PRIORITY)
 	{
 		DelListItem(module, Modules);
 		AddListItemPrio(module, Modules, action);
@@ -1335,6 +1341,16 @@ int is_module_loaded(const char *name)
 		if (mi->flags & MODFLAG_DELAYED)
 			continue; /* unloading (delayed) */
 
+		/* During config_posttest ignore modules that are loaded,
+		 * since we only care about the 'future' state.
+		 */
+		if ((loop.config_status < CONFIG_STATUS_LOAD) &&
+		    (loop.config_status >= CONFIG_STATUS_POSTTEST) &&
+		    (mi->flags == MODFLAG_LOADED))
+		{
+			continue;
+		}
+
 		if (!strcasecmp(mi->relpath, name))
 			return 1;
 	}
