@@ -43,7 +43,6 @@ EVENT(sasl_timeout);
 /* Macros */
 #define MSG_AUTHENTICATE "AUTHENTICATE"
 #define MSG_SASL "SASL"
-#define MSG_SVSLOGIN "SVSLOGIN"
 #define AGENT_SID(agent_p)	(agent_p->user != NULL ? agent_p->user->server : agent_p->name)
 
 /* Variables */
@@ -89,53 +88,6 @@ int sasl_account_login(Client *client, MessageTag *mtags)
 	return 0;
 }
 
-/*
- * SVSLOGIN message
- *
- * parv[1]: propagation mask
- * parv[2]: target
- * parv[3]: account name (SVID)
- */
-CMD_FUNC(cmd_svslogin)
-{
-	Client *target;
-
-	if (MyUser(client) || (parc < 3) || !parv[3])
-		return;
-
-	/* We actually ignore parv[1] since this is a broadcast message.
-	 * It is a broadcast message because we want ALL servers to know
-	 * that the user is now logged in under account xyz.
-	 */
-
-	target = find_client(parv[2], NULL);
-	if (target)
-	{
-		if (IsServer(target))
-			return;
-
-		if (target->user == NULL)
-			make_user(target);
-
-		strlcpy(target->user->account, parv[3], sizeof(target->user->account));
-		user_account_login(recv_mtags, target);
-		if (MyConnect(target) && IsDead(target))
-			return; /* was killed due to *LINE on ~a probably */
-	} else {
-		/* It is perfectly normal for target to be NULL as this
-		 * happens during registration phase (pre-connect).
-		 * It just means we cannot set any properties for this user,
-		 * which is fine in that case, since it will be synced via
-		 * the UID message instead.
-		 * We still have to broadcast the message, which is why
-		 * we do not return here.
-		 */
-	}
-
-	/* Propagate to the rest of the network */
-	sendto_server(client, 0, 0, NULL, ":%s SVSLOGIN %s %s %s",
-	              client->name, parv[1], parv[2], parv[3]);
-}
 
 /*
  * SASL message
@@ -370,7 +322,6 @@ MOD_INIT()
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 
 	CommandAdd(modinfo->handle, MSG_SASL, cmd_sasl, MAXPARA, CMD_USER|CMD_SERVER);
-	CommandAdd(modinfo->handle, MSG_SVSLOGIN, cmd_svslogin, MAXPARA, CMD_USER|CMD_SERVER);
 	CommandAdd(modinfo->handle, MSG_AUTHENTICATE, cmd_authenticate, MAXPARA, CMD_UNREGISTERED|CMD_USER);
 
 	HookAdd(modinfo->handle, HOOKTYPE_LOCAL_CONNECT, 0, sasl_connect);
